
// Small example how to create consistent enum/array tables
// Public domain

#include <stdio.h>

// Function definitions (simple code can be part of the table, too):
void f_ID(void) {}
void f_ID2(void) { puts("test function for ID2"); }
void f_ID3(void) {}

#undef DEF      // will be defined later

//////////////////////////////////////////////////////////////////
// Symbol table definition (definitions can be #included)
// [(enum constant, C string literal, function pointer)]
//////////////////////////////////////////////////////////////////
#define TABLE_X                         \
    DEF(ID0,"ID0",&f_ID)                \
    DEF(ID1,"ID1",&f_ID)                \
    DEF(ID2,"string for ID2",&f_ID2)    \
    DEF(ID3,"ID3",&f_ID3)               \
    DEF(ID4,"ID4",&f_ID)                \
    DEF(ID5,"ID5",&f_ID)                \

//////////////////////////////////////////////////////////////////
// End od table definition
//////////////////////////////////////////////////////////////////

// Define enum with symbols:
enum E {
#define DEF(a,b,c) a,
    TABLE_X
#undef DEF
    MAX_ID // sentinel value == number of items (can be in TABLE_X)
};

// Define string table for enum symbols:
const char * E_str[] = {        // no array size specified
#define DEF(a,b,c) b,
    TABLE_X
#undef DEF
// no need for sentinel here
};

// Define function table for enum symbols:
void (*E_fun[MAX_ID])(void) = { // array size specified
#define DEF(a,b,c) c,
    TABLE_X
#undef DEF
// sentinel here is not allowed
};

// simple test
int main() {
    enum E v = ID2;
    puts(E_str[v]);
    E_fun[v]();
    return v;
}
